/*
 * block.c -- find/create block devices and their fs nodes
 *
 * Copyright 2006-2008 Red Hat, Inc.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author(s): Peter Jones <pjones@redhat.com>
 */

#define _GNU_SOURCE 1

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <blkid/blkid.h>

#include <nash.h>
#include "lib.h"
#include "block.h"
#include "util.h"

void
block_show_labels(nashContext *c, int removables)
{
    nashBdevIter biter;
    struct nash_block_dev *dev = NULL;
    blkid_dev bdev = NULL;

    nashBlockInit(c);
    if (!(biter = nashBdevIterNew(c)))
        goto out;
    while (nashBdevIterNext(&biter, &dev) > 0) {
        blkid_tag_iterate titer;
        const char *type, *data;
        char *label=NULL, *uuid=NULL;

        if (!removables && nashBdevRemovable(dev))
            continue;

        smartmknod(dev->dev_path, S_IFBLK | 0700, dev->devno);
        bdev = blkid_get_dev(c->cache, dev->dev_path, BLKID_DEV_NORMAL);
        if (!bdev)
            continue;
        titer = blkid_tag_iterate_begin(bdev);
        while (blkid_tag_next(titer, &type, &data) >= 0) {
            if (!strcmp(type, "LABEL"))
                label = strdup(data);
            if (!strcmp(type, "UUID"))
                uuid = strdup(data);
        }
        blkid_tag_iterate_end(titer);
        if (label) {
            printf("%s %s ", dev->dev_path, label);
            free(label);
        }
        if (uuid) {
            printf("%s", uuid);
            free(uuid);
        }
        if (label)
            printf("\n");
    }
out:
    if (biter)
        nashBdevIterEnd(&biter);

    nashBlockFinish(c);
}

/*
 * vim:ts=8:sw=4:sts=4:et
 */
